@@ -5,36 +5,35 @@
  * Licensed under the GPL-2
  */
 
-#define WRAPPER_ARGS int dirfd, const char *pathname, mode_t mode
-extern int EXTERN_NAME(WRAPPER_ARGS);
-static int (*WRAPPER_TRUE_NAME)(WRAPPER_ARGS) = NULL;
+#ifndef WRAPPER_ARGS_PROTO /* let mkdir() use us */
+# define WRAPPER_ARGS_PROTO int dirfd, const char *pathname, mode_t mode
+# define WRAPPER_ARGS dirfd, pathname, mode
+# define WRAPPER_SAFE() FUNCTION_SANDBOX_SAFE_AT(dirfd, pathname)
+#endif
 
-int WRAPPER_NAME(WRAPPER_ARGS)
+static inline bool sb_mkdirat_pre_check(WRAPPER_ARGS_PROTO)
 {
-	struct stat st;
-	int result = -1, old_errno = errno;
 	char canonic[SB_PATH_MAX];
+	save_errno();
 
 	if (-1 == canonicalize(pathname, canonic))
 		/* Path is too long to canonicalize, do not fail, but just let
 		 * the real function handle it (see bug #94630 and #21766). */
 		if (ENAMETOOLONG != errno)
-			return -1;
+			return false;
 
 	/* XXX: Hack to prevent errors if the directory exist,
 	 * and are not writable - we rather return EEXIST rather
 	 * than failing */
+	struct stat st;
 	if (0 == lstat(canonic, &st)) {
 		errno = EEXIST;
-		return -1;
-	}
-	errno = old_errno;
-
-	if (FUNCTION_SANDBOX_SAFE_AT(dirfd, pathname)) {
-		check_dlsym(WRAPPER_TRUE_NAME, WRAPPER_SYMNAME,
-			    WRAPPER_SYMVER);
-		result = WRAPPER_TRUE_NAME(dirfd, pathname, mode);
+		return false;
 	}
 
-	return result;
+	restore_errno();
+	return true;
 }
+#define WRAPPER_PRE_CHECKS() if (!sb_mkdirat_pre_check(WRAPPER_ARGS)) return result;
+
+#include "__wrapper_simple.c"
